DEFAULT_TEXT_COLOR = 7

require 'helpers'

local Textarea = require 'nibui.Textarea'
local Text = require 'nibui.Text'

local Arena = {}

function Arena:new(a, b)
    return new(Arena, {
        a = a, b = b,

        terminal = Textarea:new(6, 22, 320-12, 60)
    })
end

function Arena:print(str)
    self.terminal:add(Text:new(str))
    self.terminal:newline()
end

function Arena:battle(callback)
    self.a.arena = self
    self.b.arena = self

    local turn_count = 1

    local do_turn = nil

    do_turn = function()
        self:turn(function()
            turn_count += 1

            if self:one_each_alive() then
                set_timeout(do_turn, 1)
            else
                set_timeout(callback, 2)
            end
        end)
    end

    set_timeout(do_turn, 1)
end

function Arena:turn(callback)
    self:party_attack(self.a, self.b, function()
        if self:one_each_alive() then
            self:party_attack(self.b, self.a, callback)
        else
            callback()
        end
    end)
end

function Arena:one_each_alive()
    return not self.a:all_dead() and not self.b:all_dead()
end

function Arena:party_attack(a, b, callback)
    local i = #a.party

    local function loop()
        local hero = a.party[i]

        local function next()
            a:remove_dead_heroes()
            b:remove_dead_heroes()

            if i > 1 then
                i -= 1
                set_timeout(loop, 1)
            else
                callback()
            end
        end

        if hero then
            hero:duel(hero:choose_oponent(b.party), next)
        else
            next()
        end
    end

    loop()
end

function Arena:draw(cam_x, cam_y)
    local a_x, a_y = 160, 160+700

    for i, hero in ipairs(self.b.party) do
        local x, y = (i-(1+#self.b.party)/2)*32+a_x, a_y-32
        hero:draw(cam_x+x, cam_y+y)
    end

    for i, hero in ipairs(self.a.party) do
        local x, y = (i-(1+#self.a.party)/2)*32+a_x, a_y+32-16
        hero:draw(cam_x+x, cam_y+y)
    end

    self.terminal:draw()
end

return Arena
