require 'helpers'

local Body = {
    Idle = 1,
    Attacking = 2,
    Hit = 3,
    Missed = 4,
    Hurt = 5,
    Evaded = 6,
}

function Body:new(hero, human_player)
    return new(Body, {
        back = human_player or false,
        hero = hero,
        state = Body.Idle,
        frame = 0,
        frame_randomizer = math.random(0, 2),
        dx = 0,
        dy = 0,
    })
end

function Body:attack()
    if is_in(self.state, {Body.Idle}) then
        self.state = Body.Attacking
    end
end

function Body:hit()
    if is_in(self.state, {Body.Attacking}) then
        self.state = Body.Hit
    end
end

function Body:missed()
    if is_in(self.state, {Body.Attacking}) then
        self.state = Body.Missed
    end
end

function Body:hurt()
    if is_in(self.state, {Body.Idle}) then
        self.state = Body.Hurt
    end
end

function Body:evaded()
    if is_in(self.state, {Body.Idle}) then
        self.state = Body.Evaded
    end
end

function Body:idle()
    if is_in(self.state, {Body.Hurt, Body.Hit, Body.Missed, Body.Evaded}) then
        self.state = Body.Idle
    end
end

function Body:sprite()
    local characters = {
        [0] = {
            vitality = {
                [false] = {15, 2},
                [true] = {16, 2}
            },
            strength = {
                [false] = {15, 2},
                [true] = {16, 2}
            },
            agility = {
                [false] = {15, 2},
                [true] = {16, 2}
            },
        },
        [1] = {
            vitality = {
                [false] = {17, 3},
                [true] = {18, 3}
            },
            strength = {
                [false] = {21, 4},
                [true] = {22, 4}
            },
            agility = {
                [false] = {25, 5},
                [true] = {26, 5}
            },
        },
        [2] = {
            vitality = {
                [false] = {19, 3},
                [true] = {20, 3}
            },
            strength = {
                [false] = {23, 4},
                [true] = {24, 4}
            },
            agility = {
                [false] = {27, 5},
                [true] = {28, 5}
            },
        }
    }

    local level = math.min(math.floor(self.hero:points()/20), 2)
    local high = self.hero:high_stat()

    local spr, pal = unwrap(characters[level][high][self.back] or {15, 2})

    return spr, pal
end

function Body:draw(x, y)
    if self.frame == 1 then
        self.dx += math.cos(clock()*2+self.frame_randomizer)*1/3
        self.dy += math.sin(clock()*2+self.frame_randomizer)*1/10
    end

    local spr, pal = self:sprite()

    sprite(x+self.dx,
           y+self.dy,
           7+self.frame, spr, pal)

    if is_in(self.state, {Body.Attacking, Body.Hit, Body.Missed}) then
        custom_sprite(x+self.dx-3, y-20-math.sin(clock()*16)*4, 144, 240, 22, 18)
    elseif self.state == Body.Evaded then
        custom_sprite(x+self.dx-3, y-20-math.sin(clock()*16)*4, 144+22, 240, 22, 18)
    elseif self.state == Body.Hurt then
        custom_sprite(x+self.dx-3, y-20-math.sin(clock()*16)*4, 144+22*2, 240, 22, 18)
    end

    self:draw_hp(x+self.dx-8, y+self.dy+18)

    if self.state ~= Body.Idle then
        print(self.hero.name, x+8-measure(self.hero.name)/2, y+24)
    end

    self.frame = math.floor(clock()*2+self.frame_randomizer)%2;
end

function Body:draw_hp(x, y)
    local ratio = self.hero.vitality/self.hero.vitality_cap
    local sy = math.ceil(ratio*6)*6 + 122

    custom_sprite(x, y, 0, sy, 32, 6)
end

return Body
