HEADS = true
TAILS = false

function coin_flip()
    return math.random() >= 0.5;
end

function new(kind, obj)
    return instanceof(obj, kind);
end

function first(table)
    if not table or #table == 0 then
        return nil
    else
        return table[1]
    end
end

function last(table)
    if not table or #table == 0 then
        return nil
    else
        return table[#table]
    end
end

function is_in(value, table)
    for _, v in ipairs(table) do
        if v == value then
            return true
        end
    end

    return false
end

-- https://stackoverflow.com/questions/49709998/how-to-filter-a-lua-array-inplace#49709999
function filter_inplace(arr, func)
    for i=#arr,1,-1 do
        if not func(arr[i]) then
            remove(arr, i)
        end
    end
end

function dim(color)
    return ({
        0, 1, 2, 3, 4, 5, 6, 3, 2, 6, 7, 5, 9, 8, 12, 14
    })[color+1]
end

-- Kerning-capable print
local DEFAULT_FT_W = 10
local DEFAULT_FT_H = 10
local DEFAULT_CH_W = 8
local DEFAULT_CH_H = 8
local DEFAULT_FT_CHARS = "1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZ.,!?abcdefghijklmnopqrstuvwxyz()[]<>{}\"\"'-_=\\/|&~*%@$#:;+"..from_ascii(1)..from_ascii(2)..from_ascii(3)..from_ascii(4)..from_ascii(5)..from_ascii(6)..from_ascii(7)..from_ascii(8)..from_ascii(9)..from_ascii(10)..from_ascii(11)..from_ascii(12)..from_ascii(13)..from_ascii(14)..from_ascii(15)..from_ascii(16)..from_ascii(17).." "
local DEFAULT_KERNS = {}

function measure(str)
  local len = 0

  for i=1,#str do
    local ch = str:sub(i, i)
    local index, _ = DEFAULT_FT_CHARS:find(ch, 1, true)

    if index ~= nil and DEFAULT_KERNS[index] then
      len += DEFAULT_KERNS[index][3]
    end
  end

  return len
end

function print(str, dstx, dsty, pal)
    for i=1,#str do
        local ch = str:sub(i, i)
        local index, _ = DEFAULT_FT_CHARS:find(ch, 1, true)

        if index ~= nil and DEFAULT_KERNS[index] then
            local px = DEFAULT_KERNS[index][1]
            local py = DEFAULT_KERNS[index][2]

            custom_sprite(dstx, dsty+DEFAULT_KERNS[index][5], px, py, DEFAULT_KERNS[index][3], DEFAULT_KERNS[index][4], pal)

            dstx = dstx+DEFAULT_KERNS[index][3]
        end
    end
end

function has_color(base_x, base_y)
    for y=0,7 do
        local pixel_a = get_sheet_pixel(base_x, base_y+y)

        if pixel_a == 6 or pixel_a == 7 then
            return true
        end
    end

    return false
end

function has_vcolor(base_x, base_y, size)
    for x=0,size do
        local pixel_a = get_sheet_pixel(base_x+x, base_y)

        if pixel_a == 6 or pixel_a == 7 then
            return true
        end
    end

    return false
end

function find_next_stop(x, y)
    local i = 1

    while i < 8 do
        if has_color(x, y) then
            x += 1
            i = 1
        else
            if i >= 3 then
                break
            else
                x += 1
            end
        end

        i += 1
    end

    return x, y
end

function find_next_vstop(x, y, size)
    for i=1,10 do
        if has_vcolor(x, y, size) then
            y += 1
        else
            if i >= 5 then
                break
            else
                y += 1
            end
        end
    end

    return x, y+1
end

function find_next_vstart(x, y, size)
    for i=1,10 do
        if has_vcolor(x, y, size) then
            break
        else
            y += 1
        end
    end

    return x, y-1
end

function init_kerning_font()
    local x, y = 0, 0

    local chars = {}

    local baselines = {}
    
    for l=1,11 do
        x = 0

        insert(baselines, 0)

        insert(chars, {x, y})

        for c=1,10 do
            x, y = find_next_stop(x, y)
            insert(chars , {x, y})
        end

        x, y = find_next_vstop(0, y, 71)
    end

    for i, char in ipairs(chars) do
        if i%11 ~= 0 and i < #chars then
            local sx, sy = find_next_vstart(char[1], char[2], chars[i+1][1]-char[1])
            local ex, ey = find_next_vstop(sx, sy, chars[i+1][1]-char[1])

            local w, h = chars[i+1][1]-char[1], chars[math.min(i+11, #chars)][2]-char[2]

            baselines[math.min(math.floor((i-1)/11+1), #baselines)] += (ey-char[2])/11
        end
    end

    for i, char in ipairs(chars) do
        if i%11 ~= 0 and i < #chars then
            local sx, sy = find_next_vstart(char[1], char[2], chars[i+1][1]-char[1])
            local ex, ey = find_next_vstop(sx, sy, chars[i+1][1]-char[1])

            local w, h = chars[i+1][1]-char[1], chars[math.min(i+11, #chars)][2]-char[2]

            insert(DEFAULT_KERNS, {
                char[1], char[2],
                w, h,
                7-math.floor(baselines[math.min(math.floor((i-1)/11+1), #baselines)]),
            })
        end
    end

    local spc, _ = DEFAULT_FT_CHARS:find(' ', 1, true)

    DEFAULT_KERNS[spc] = {64, 32, 6, 6, 0}
end
