require 'helpers'

local Body = require 'body'

local Hero = {}

function Hero:new(player)
    local names = require 'hero_names_small'
    local titles = require 'hero_titles'

    local name = names[math.floor(math.random()*#names)+1]
    local title = titles[math.floor(math.random()*#titles)+1]

    if math.random() < 1/666 then
        name = 'Lucia'
        title = 'Rubber Dick'
    end

    local hero = new(Hero, {
        name =  name,
        title = '"'..title..'"',

        vitality = 0, vitality_cap = 5,
        strength = 0, strength_cap = 5,
        agility = 0, agility_cap = 5,

        player = player
    })

    hero.body = Body:new(hero, player.human)

    return hero
end

function Hero:change_stat(stat, amount)
    if self[stat]+amount <= self[stat..'_cap'] and self[stat]+amount >= 0 then
        if self.player.vitality >= amount then
            self[stat] += amount
            self.player.vitality -= amount
        end
    end
end

function Hero:choose_oponent(oponents)
    if oponents == nil or #oponents == 0 then
        return nil
    end

    return oponents[math.random(1, #oponents)]
end

function Hero:duel(enemy_hero, callback)
    if enemy_hero == nil then
        callback()
        return
    end

    self:attack(enemy_hero, callback)
end

function Hero:hit_chance(enemy_hero)
    local agility = self.agility
    local enemy_agility = enemy_hero.agility

    if agility == enemy_agility then
        return 0.5
    end

    local diff = enemy_agility-agility

    if diff <= 0 then
        return (1+(1-(-1/diff)))/2
    else
        return (1/diff)/2
    end
end

function Hero:attack(enemy_hero, callback)
    local hit_chance = self:hit_chance(enemy_hero)

    self.body:attack()

    local wait_time = 2

    if math.random() <= hit_chance+0.25 then
        set_timeout(function ()
            local friends_bonus = math.max(0, #self.player.party-#enemy_hero.player.party)+1

            local attack_strength = math.min(self.strength*friends_bonus+1, enemy_hero.vitality)

            enemy_hero.vitality -= attack_strength

            if self.player then
                self.player.vitality += attack_strength
            end

            self.player.arena:print(self.name..' hit '..enemy_hero.name..' ('..tostring(attack_strength)..')')

            if enemy_hero:dead() then
                self.player.arena:print(enemy_hero.name..' died')
            end

            self.body:hit()
            enemy_hero.body:hurt()

            set_timeout(function()
                self.body:idle()
                enemy_hero.body:idle()
                callback()
            end, wait_time)
        end, wait_time)
    else
        set_timeout(function ()
            self.body:missed()
            enemy_hero.body:evaded()

            self.player.arena:print(self.name..' missed '..enemy_hero.name)

            set_timeout(function()
                self.body:idle()
                enemy_hero.body:idle()
                callback()
            end, wait_time)
        end, wait_time)
    end
end

function Hero:dead()
    return self.vitality <= 0
end

function Hero:increase_cap()
    for _, prop in ipairs({'vitality', 'strength', 'agility'}) do
        if self[prop] == self[prop..'_cap'] then
            self[prop..'_cap'] = self[prop..'_cap']*2
        end
    end
end

function Hero:points()
    local acc = 0

    for _, prop in ipairs({'vitality_cap', 'strength_cap', 'agility_cap'}) do
        acc += self[prop]
    end

    return acc
end

function Hero:high_stat()
    local stats = {
        { 'vitality', self.vitality_cap },
        { 'strength', self.strength_cap },
        { 'agility', self.agility_cap },
    }

    sort(stats, function(a, b) return a[2] > b[2] end)

    return stats[1][1]
end

function Hero:draw(x, y)
    self.body:draw(x, y)
end

return Hero
