local music = require 'music'

local SceneManager = require 'scene_manager'
local IntroScene = require 'scenes.intro'

local Player = require 'player'

local scene_manager = nil

local timeouts = {}

function set_timeout(fn, time)
    push(timeouts, {fn, clock()+time})
end

function init()
    init_kerning_font()

    -- UI Palette
    write(0, '\x00\x00\x00\x00\x40\x24\x2d\xff\x61\x2e\x2e\xff\x7d\x40\x32\xff\x99\x51\x39\xff\xb5\x65\x3a\xff\xeb\x96\x5e\xff\xff\xd1\xb3\xff\xb0\x20\x20\xff\xbd\x48\x48\xff\xe3\x9a\x9a\xff\xa5\x84\xb0\xff\x72\x5a\x8a\xff\x68\x30\x78\xff\x60\x6e\x3c\xff\x76\xa6\x5a\xff')
    -- Tower scene palette
    write(64, '\x00\x00\x00\xff\xc4\x81\x5f\xff\x9a\x5d\x49\xff\x5b\x3b\x31\xff\x3c\x37\x3a\xff\x53\x47\x4a\xff\x61\x4e\x4e\xff\x71\x5e\x55\xff\x7d\x6d\x61\xff\x88\x7c\x64\xff\x3f\x6a\x4b\xff\x4b\x7a\x3b\xff\x5a\xa6\xf0\xff\x7e\xc3\xf4\xff\xb6\xdc\xf7\xff')
    -- Character palettes
    -- 1. Base
    write(64*2, '\x00\x00\x00\x00\x76\x4d\x4d\xff\x8f\x5f\x50\xff\x40\x24\x2d\xff\x61\x2e\x2e\xff\x7d\x40\x32\xff\x9c\x5b\x35\xff\xc4\x76\x4d\xff\x3c\x40\x40\xff\x59\x5c\x5a\xff\x6f\x78\x76\xff\x88\xa1\xb3\xff\x78\x25\x12\xff\x9e\x23\x13\xff\xd9\x16\x00\xff\xf2\xbf\x9d\xff')
    -- 2. High HP
    write(64*3, '\x00\x00\x00\x00\x76\x4d\x4d\xff\x8f\x5f\x50\xff\x40\x24\x2d\xff\x61\x2e\x2e\xff\x7d\x40\x32\xff\x9c\x5b\x35\xff\xc4\x76\x4d\xff\x66\x2d\x2e\xff\x85\x36\x36\xff\xab\x58\x49\xff\xd4\x7f\x6a\xff\x78\x25\x12\xff\x9e\x23\x13\xff\xd9\x16\x00\xff\xf2\xbf\x9d\xff')
    -- 3. High STR
    write(64*4, '\x00\x00\x00\x00\x76\x4d\x4d\xff\x8f\x5f\x50\xff\x40\x24\x2d\xff\x61\x2e\x2e\xff\x7d\x40\x32\xff\x9c\x5b\x35\xff\xc4\x76\x4d\xff\x45\x34\x43\xff\x5f\x44\x66\xff\x67\x65\x8f\xff\x8f\x95\xbd\xff\x78\x25\x12\xff\x9e\x23\x13\xff\xd9\x16\x00\xff\xf2\xbf\x9d\xff')
    -- 4. High AGIL 
    write(64*5, '\x00\x00\x00\x00\x76\x4d\x4d\xff\x8f\x5f\x50\xff\x40\x24\x2d\xff\x61\x2e\x2e\xff\x7d\x40\x32\xff\x9c\x5b\x35\xff\xc4\x76\x4d\xff\x34\x45\x40\xff\x4e\x66\x44\xff\x65\x8f\x75\xff\x8f\xbd\xb1\xff\x78\x25\x12\xff\x9e\x23\x13\xff\xd9\x16\x00\xff\xf2\xbf\x9d\xff')

    mask_color(0)
    mask_color(16)
    mask_color(32)
    mask_color(48)
    mask_color(64)
    mask_color(80)
    math.randomseed(time())

    scene_manager = SceneManager:new()

    scene_manager:start_scene(IntroScene:new())

    music.init()
    --start_capturing('gifs/ui-4.gif')
end

function draw()
    scene_manager:draw()
end

function update(dt)
    scene_manager:update(dt)

    for i=#timeouts,1,-1 do
        local fn, time = timeouts[i][1], timeouts[i][2]

        if time <= clock() then
            remove(timeouts, i)
            fn()
        end
    end
end
