require 'helpers'

local Hero = require 'hero'

local Player = {}

function Player:new(vitality, human)
    return new(Player, {
        vitality = vitality or 4,
        max_party_size = 14,
        human = human,
        party = {}
    })
end

function Player:party_full()
    return self.max_party_size == #self.party
end

function Player:create_hero()
    local cost = 1

    if self.vitality >= cost then
        if #self.party < self.max_party_size then
            self.vitality -= cost 
            local hero = Hero:new(self)
            insert(self.party, hero)

            return hero
        else
            return nil
        end
    else
        return nil
    end
end

function Player:remove_dead_heroes()
    filter_inplace(self.party, function(hero)
        return not hero:dead()
    end)
end

function Player:all_dead()
    local all = true

    for _, hero in ipairs(self.party) do
        if not hero:dead() then
            all = false
        end
    end

    return all
end

function Player:prepare_heroes_randomly()
    while self.vitality > 0 do
        if math.random() < 0.05 or #self.party == 0 then
            self:create_hero():change_stat('vitality', 1)
        end

        if #self.party > 0 then
            local hero = self.party[math.floor(math.random()*#self.party)+1]
            local prop = ({'vitality', 'strength', 'agility'})[math.floor(math.random()*3)+1]

            hero:change_stat(prop, 1)
        else
            break
        end

        if math.random() <= 0.5 then
            self:raise_caps()
        end
    end
end

function Player:raise_caps()
    for _, hero in ipairs(self.party) do
        hero:increase_cap()
    end
end

function Player:all_vitality_points()
    local vitality = self.vitality

    for _, hero in ipairs(self.party) do
        vitality += hero.vitality
        vitality += hero.strength
        vitality += hero.agility
    end

    return vitality
end

return Player
