local Easing = require 'nibui.Easing'
local NOM = require 'nibui.NOM'
require 'nibui.Neact'

local HeroCard = Neact.Component:new()

local CardTitle = require 'ui.card_title'
local CardPicture = require 'ui.card_picture'
local CardStats = require 'ui.card_stats'

local BG_CARD = {80, 16, 32, 32, 10, 32-10, 10, 32-10}
local CARD_W = 128
local CARD_H = 188

function HeroCard:new(props)
   return new(HeroCard, {
                 closed = true,
                 state = {
                    title = props.hero.name .. ' ' .. props.hero.title
                 },
                 props = props,
   })
end

function HeroCard:close()
   if not self.closed then
      self.card.x = {self.props.x, 0.2, Easing.Linear}
      self.card.y = {self.props.y, 0.2, Easing.Linear}

      self.card.w = {CARD_W, 0.2, Easing.Linear}
      self.card.h = {CARD_H, 0.2, Easing.Linear}

      self.closed = true
   end
end

function HeroCard:open()
   if self.closed then
      local pad = 6
      local pad_top = 22

      self.props.manager:close_all()

      self.card.x = {NOM.left+NOM.parent.w/2+pad, 0.2, Easing.Linear}
      self.card.y = {NOM.top+pad_top, 0.2, Easing.Linear}

      self.card.w = {NOM.parent.w/2-2*pad, 0.2, Easing.Linear}
      self.card.h = {NOM.parent.h-22-pad-pad_top, 0.2, Easing.Linear}

      self.closed = false
   end
end

function HeroCard:render(state, props)
   local pad = 4

   local hover_movement = 12

   return {
      ref = function(card) self.card = card end,

      x = props.x, y = props.y, w = CARD_W, h = CARD_H,
      background = BG_CARD,

      closed = true,

      onenter = function(el)
          if self.closed then
              el.y = {props.y-hover_movement, 0.1, Easing.OutCubic}
          end
      end,
      onleave = function(el)
          if self.closed then
              el.y = {props.y, 0.2, Easing.InCubic}
          end
      end,
      onclick = function(el)
         if self.closed then
            self:open()
         else
            self:close()
         end

         return true
      end,

      {
         x = NOM.left+pad, y = NOM.top+pad,
         w = NOM.parent.w-2*pad, h = NOM.parent.h-2*pad,

         {CardTitle, title = state.title},
         {CardPicture, picture = nil},
         {CardStats, hero = props.hero}
      }
   }
end

return HeroCard
