local NOM = require 'nibui.NOM'
local Widget = require 'nibui.Widget'
require 'nibui.Neact'

local ProgressBar = Neact.Component:new()

local PROGRESS_BAR = {80, 50, 48, 13, 4, 44, 7, 8}
local PROGRESS = {89, 3, 8, 0, 3, 2, 3}

function ProgressBar:new(props)
   return new(ProgressBar, {
                 props = props,
                 state = {
                    mode = props.mode or 'percent'
                 }
   })
end

function ProgressBar:render(state, props)
   local pad = 2

   local function ratio()
       local table, key, max = props.table, props.key, props.max

       local str_ratio = tostring(table[key])..'/'..tostring(table[max])

       if table[max] == 0 then
          return table[max], str_ratio
       else
          return table[key]/table[max], str_ratio
       end
   end

   return {
      x = NOM.left, y = NOM.top, w = NOM.parent.w, h = NOM.parent.h,
      background = PROGRESS_BAR,

      {
         x = NOM.left, y = NOM.top,
         h = NOM.parent.h, w = NOM.parent.w,

         padding_bottom = 1,

         content = NOM.fn(function(self)
               if state.mode == 'percent' then
                  local ratio = math.floor(ratio()*1000)/10

                  return tostring(ratio)..'%'
               else
                  local _, str = ratio()
                  return props.key:sub(1, 3):upper()..' '..str
               end
         end),
      },

      {
        x = NOM.left+pad, y = NOM.top+pad,
        h = NOM.parent.h-2*pad-1,
        w = NOM.fn(function(self)
              return (self.parent.w-2*pad)*math.min(ratio(), 1)
        end),

        background = { props.color*2, unwrap(PROGRESS) },
      }
   }
end

return ProgressBar
