local audio = {}

-- Converte uma string binária em um número,
-- considerando a string como MSB
local function msb(str, bits_per_byte)
    bits_per_byte = bits_per_byte or 8

    if #str > 0 then
        local mask = math.pow(2, bits_per_byte)-1
        local mul = math.pow(2, (#str-1)*bits_per_byte)

        local value = bit.band(str:byte(), mask)

        return value*mul+msb(str:sub(2), bits_per_byte)
    else
        return 0
    end
end

local function byte(data)
    local byte = data.data:sub(data.ptr, data.ptr):byte()

    data.ptr += 1

    return byte
end

local function varlen(data)
    local buf = ''
    local byte

    repeat
        byte = byte(data)

        buf = buf..byte
    until byte <= 127

    return msb(buf, 7)
end

local function read_chunk(file)
    local kind = file:read(4)

    if not kind then
        return nil
    end

    local length = msb(file:read(4))
    local data = file:read(length)

    return {
        kind = kind,
        length = length,
        data = {
            data = data,
            ptr = 1,
        }
    }
end

local function read_track(data)
    local events = {}
    local status

    repeat
        local delta = varlen(data)
        local tmp_status = byte(data)

        -- Usaremos o satus anterior
        if bit.band(tmp_status, 0x80) == 0 then
            data.ptr -= 1
        else
            status = tmp_status
        end

        if status == 0xff then
            local _ = byte(data)
            data.ptr += varlen(data)
        elseif status == 0xf0 then
            repeat
            until byte(data) == 0xf7
        elseif ({[0xd0] = true, [0xc0] = true})[bit.band(status, 0xf0)] then
        elseif ({[0x80] = true, [0x90] = true})[bit.band(status, 0xf0)] then
            local note = byte(data)
            local velocity = byte(data)

            if velocity == 0 then
                status = 0x80
            end

            insert(events, {
                       delta = delta,
                       ch = bit.band(status, 0x0f),
                       on = ({[0x90] = true})[bit.band(status, 0xf0)],
                       note = note,
                       velocity = velocity,
            })
        else
            data.ptr += 2
        end
    until data.ptr >= #data.data

    return events
end

function audio.read_music(file)
    local tracks = {}

    --while true do
    --    local chunk = read_chunk(file)

    --    if not chunk then
    --        break
    --    else
    --        if chunk.kind == 'MTrk' then
    --            local events = read_track(chunk.data)

    --            if #events > 0 then
    --                insert(tracks, events)
    --            end
    --        end
    --    end
    --end

    return tracks
end

return audio
